<?xml version="1.0"?>
<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:lxslt="http://xml.apache.org/xslt"
    xmlns:corianderpy="corianderpy"
    extension-element-prefixes="corianderpy"
    >

  <lxslt:component prefix="corianderpy" functions="doCreateImage">
<lxslt:script lang="jpython">
<![CDATA[

# ----------------------------------------------------------------------------
#
# This code should all be in a separate file... 
# as soon as I work out how paths work in Jython imports
#
# ----------------------------------------------------------------------------

from inigo.java2d import JLayer

from java.io import *
from com.sun.image.codec.jpeg import *

from java.awt import *
from java.awt.color import *
from java.awt.geom import *
from java.awt.image import *
from java.awt.event import *

# The PPaint class encapsulates a Java Paint - for example, Color or GradientPaint
# Remember that wherever Color is being used, it's possible to use its full constructor
# Color(r, g, b, alpha) to get alpha blending in the PPaint as well

class PPaint:
    "A paint type for a single color"
    def __init__(self, col = Color.black):
        self.myPaint = col
    # This is used as a callback by layer, so the PPaint can calculate
    # layer-specific parameters... for example, see PVGradientPaint
    def applyPaint(self, layer):
        layer.myGraphics.paint = self.myPaint

class PGradientPaint(PPaint):
    "A paint type for a graduated fill"
    def __init__(self, x, y, col1, dx, dy, col2):
        self.myPaint = GradientPaint(x, y, col1, dx, dy, col2)

class PVGradientPaint(PGradientPaint):
    "A paint type for a graduated vertical fill"
    def __init__(self, col1, col2):
        PGradientPaint.__init__(self, 0, 0, col1, 0, 40, col2)
        self.col1 = col1
        self.col2 = col2
    def applyPaint(self, layer):
        height = layer.myJLayer.image.height
        myPaint = GradientPaint(0,0, self.col1, 0, height, self.col2)
        layer.myGraphics.paint = self.myPaint

class PHGradientPaint(PVGradientPaint):
    "A paint type for a graduated horizontal fill"
    def applyPaint(self, layer):
        print "Calling applyPaint for H"
        width = layer.myJLayer.image.width
        myPaint = GradientPaint(0,0, self.col1, width, 0, self.col2)
        layer.myGraphics.paint = self.myPaint


class PLayer:
    # Constructor creates the associated Java JLayer
    def __init__(self, width=200, height=200):
        # This is method overloading - and is particularly nasty
        # @todo Find a better way
        try:
            newWidth = width.myJLayer.image.width
            newHeight = width.myJLayer.image.height
        except AttributeError:
            newWidth = width
            newHeight = height
        self.myJLayer = JLayer(newWidth,newHeight)
        # Using the JavaBean integration - image is using getImage()
        # Likewise, an assignation would use setImage(img)
        self.myGraphics = self.myJLayer.image.createGraphics()
        self.setAlpha(1)
        self.setPaint(Color.black)
        # Note we don't turn on anti-aliasing here - otherwise it causes problems when the layers merge
        # Actually, we are now - there are less problems if we use anti-aliasing here rather than later
        self.myGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
    # Operator overloading the + to merge two layers together
    def __add__(self, other):
        width = max(self.myJLayer.image.width, other.myJLayer.image.width)
        height = max(self.myJLayer.image.height, other.myJLayer.image.height)
        union = PLayer(width, height)
        # Turn on antialiasing for the merge
        union.myGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
        union.myGraphics.drawImage(self.myJLayer.image, 0, 0, None)
        union.myGraphics.drawImage(other.myJLayer.image, 0, 0, None)
        return union
    # Set to be the main canvas in the Java Swing component
    def show(self):
        local.setImg(self.myJLayer.image)

    # Output text using the current font
    def text(self, textToDraw, x=50, y=50):
        self.myGraphics.drawString(textToDraw, x,y)
    # Output text using the current font with a shadow
    def shadowText(self, textToDraw, x=50, y=50, shadowDx=3, shadowDy=3, shadowOpacity=0.4):
        oldAlpha = self.myAlpha
        oldPaint = self.myPaint
        # The shadow is 40% as opaque as current opacity, by default
        self.myGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, oldAlpha*shadowOpacity))
        # The shadow should always be black... is this reasonable?
        self.myGraphics.paint=Color.black
        self.myGraphics.drawString(textToDraw, x+shadowDx,y+shadowDy)
        # And restore the old settings before drawing the actual text over the top
        self.setAlpha(oldAlpha)
        self.setPaint(oldPaint)
        self.myGraphics.drawString(textToDraw, x,y)
    # Fill with a blank colour or paint
    def fill(self, col=Color.white):
        self.setPaint(col)
        rec = Rectangle2D.Float(0,0,self.myJLayer.image.width,self.myJLayer.image.height)
        self.myGraphics.fill(rec)

    # Sets the font to be used for drawing
    def setFont(self, font):
        # Override the current paint, only if there is one set in the font
        if (font.getPaint()!=None):
            self.setPaint( font.getPaint() )
        self.myGraphics.font = font.getFont()
    # Takes an alpha channel value between 0 and 1
    def setAlpha(self, fAlpha):
        self.myAlpha = fAlpha
        self.myGraphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, fAlpha))
    # Merely a convenient name - duplicates setPaint
    def setColor(self, newPaint = Color.black):
        self.setPaint(newPaint)
    # Sets the Paint object to draw subsequent items with
    def setPaint(self, newPaint = Color.black):
        self.myPaint = newPaint
        # This allows the parameter to be either a Java Color/Paint, or a Python PPaint
        try:
            newPaint.applyPaint(self)
        except AttributeError:
            self.myGraphics.paint = newPaint

    def loadImage(self, filename):
        image = self.myJLayer.loadImage(filename)
        self.myJLayer.image = image
        self.myGraphics = self.myJLayer.image.createGraphics()
        self.myGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
    def saveImage(self):
        return self.myJLayer.saveImage()


class PFont:
    def __init__(self, face="Arial", size = 40, weight = Font.PLAIN):
        self.myFont = Font(face, weight, size)
    def getFont(self):
        return self.myFont
    def setColor(self, col):
        self.myPaint = col
    def getPaint(self):
        # @todo Find a better way of doing this without the exceptions
        try:
            return self.myPaint
        except AttributeError:
            return None

# ----------------------------------------------------------------------------

def doCreateImage(title):
  f1 = PFont("Arial", 20)
  f1.setColor(Color.red)
  vPaint = PVGradientPaint(Color.green, Color.yellow)
  l = PLayer(120,30)
  l.fill(vPaint)
  l.setFont(f1)
  l.shadowText( title, 10, 20 )
  filePath = l.saveImage()
  fileName = filePath[max(("/"+filePath).rindex("/"), ("/"+filePath).rindex("\\")):]
  return fileName

]]>
</lxslt:script>
  </lxslt:component>

	<xsl:template match="item">
		<p>
		<img>
			<xsl:attribute name="src">generated-images/<xsl:value-of select="string(corianderpy:doCreateImage(string(text())))"/></xsl:attribute>
		</img>
		</p>
	</xsl:template>
	
	<xsl:template match="title">	
		<h1>
			<xsl:apply-templates />
		</h1>
	</xsl:template>

	<xsl:template match="body">	
		<p>
			<xsl:apply-templates />
		</p>
	</xsl:template>
</xsl:stylesheet>
